@@ -50,127 +50,115 @@ json_value_t* json_value_copy(const json_value_t* val)
 
 } // namespace
 
-Json::Json() : node_(json_value_create(JSON_VALUE_NULL))
-{
-}
-
-Json::Json(const std::string& str, bool parse_flag)
-    : node_(json_value_parse(str.c_str()))
+// ------------------------ Constructor -------------------------
+Json::Json()
+    : node_(json_value_create(JSON_VALUE_NULL)), parent_(nullptr),
+      allocated_(true)
 {
 }
 
 Json::Json(const std::string& str)
-    : node_(json_value_create(JSON_VALUE_STRING, str.c_str()))
+    : node_(json_value_create(JSON_VALUE_STRING, str.c_str())),
+      parent_(nullptr), allocated_(true)
 {
 }
 
-Json::Json(const char* str) : node_(json_value_create(JSON_VALUE_STRING, str))
+Json::Json(const char* str)
+    : node_(json_value_create(JSON_VALUE_STRING, str)), parent_(nullptr),
+      allocated_(true)
 {
 }
 
-Json::Json(std::nullptr_t null) : node_(json_value_create(JSON_VALUE_NULL))
-{
-}
-
-Json::Json(double val) : node_(json_value_create(JSON_VALUE_NUMBER, val))
-{
-}
-
-Json::Json(int val)
-    : node_(json_value_create(JSON_VALUE_NUMBER, static_cast<double>(val)))
+Json::Json(std::nullptr_t null)
+    : node_(json_value_create(JSON_VALUE_NULL)), parent_(nullptr),
+      allocated_(true)
 {
 }
 
 Json::Json(bool val)
     : node_(val ? json_value_create(JSON_VALUE_TRUE)
-                : json_value_create(JSON_VALUE_FALSE))
+                : json_value_create(JSON_VALUE_FALSE)),
+      parent_(nullptr), allocated_(true)
 {
 }
 
-// todo : optimize
-Json::Json(const Array& val)
-    : node_(json_value_copy(val.node_)), parent_(val.parent_),
-      parent_key_(val.parent_key_)
+Json::Json(const std::vector<std::string>& val)
+    : node_(json_value_create(JSON_VALUE_ARRAY)),
+      parent_(nullptr), allocated_(true)
 {
+    json_array_t* arr = json_value_array(node_);
+    for (const auto& str : val)
+    {
+        json_array_append(arr, JSON_VALUE_STRING, str.c_str());
+    }
 }
 
-Json::Json(const Object& val)
-    : node_(json_value_copy(val.node_)), parent_(val.parent_),
-      parent_key_(val.parent_key_)
+// for parse
+Json::Json(const std::string& str, bool parse_flag) : parent_(nullptr)
 {
-}
-
-Json::Json(Array&& val)
-    : node_(val.node_), parent_(val.parent_),
-      parent_key_(std::move(val.parent_key_))
-{
-    val.node_ = nullptr;
-    val.parent_ = nullptr;
-}
-
-Json::Json(Object&& val)
-    : node_(val.node_), parent_(val.parent_),
-      parent_key_(std::move(val.parent_key_))
-{
-    val.node_ = nullptr;
-    val.parent_ = nullptr;
+    node_ = json_value_parse(str.c_str());
+    allocated_ = node_ == nullptr ? false : true;
 }
 
 Json::~Json()
 {
-    if (allocated_)
-    {
-        destroy_node(node_);
-    }
+    destroy_node(&node_);
 }
 
 // watcher constructor
-Json::Json(const json_value_t* parent, std::string&& key)
-    : node_(json_value_create(JSON_VALUE_NULL)), parent_(parent),
-      parent_key_(std::move(key)), allocated_(false)
-{
-}
-
-Json::Json(const json_value_t* parent, const std::string& key)
-    : node_(json_value_create(JSON_VALUE_NULL)), parent_(parent),
-      parent_key_(key), allocated_(false)
-{
-}
-
-Json::Json(const json_value_t* parent)
-    : node_(json_value_create(JSON_VALUE_NULL)), parent_(parent),
-      allocated_(false)
-{
-}
-
-Json::Json(const json_value_t* node, const json_value_t* parent)
-    : node_(node), parent_(parent), allocated_(false)
-{
-}
-
 Json::Json(const json_value_t* node, const json_value_t* parent,
            std::string&& key)
-    : node_(node), parent_(parent), parent_key_(std::move(key)),
-      allocated_(false)
+    : node_(const_cast<json_value_t*>(node)), parent_(parent),
+      allocated_(false), parent_key_(std::move(key))
 {
 }
 
 Json::Json(const json_value_t* node, const json_value_t* parent,
            const std::string& key)
-    : node_(node), parent_(parent), parent_key_(key), allocated_(false)
+    : node_(const_cast<json_value_t*>(node)), parent_(parent),
+      allocated_(false), parent_key_(key)
 {
 }
 
-Json::Json(const Empty&)
+Json::Json(JsonType type) : allocated_(true)
 {
+    if (type == JsonType::Object)
+    {
+        node_ = json_value_create(JSON_VALUE_OBJECT);
+    }
+    else
+    {
+        node_ = json_value_create(JSON_VALUE_ARRAY);
+    }
+}
+Json::Json(const Json& other)
+{
+    node_ = json_value_copy(other.node_);
+    allocated_ = true;
+}
+
+Json& Json::operator=(const Json& other)
+{
+    if (this == &other)
+    {
+        return *this;
+    }
+    destroy_node(&node_);
+    node_ = json_value_copy(other.node_);
+    parent_ = nullptr;
+    allocated_ = true;
+    parent_key_.clear();
+
+    return *this;
 }
 
 Json::Json(Json&& other)
-    : node_(other.node_), parent_(other.parent_),
-      parent_key_(std::move(other.parent_key_)), allocated_(other.allocated_)
+    : node_(other.node_), parent_(other.parent_), allocated_(other.allocated_),
+      parent_key_(std::move(other.parent_key_))
 {
     other.node_ = nullptr;
     other.parent_ = nullptr;
+    other.allocated_ = false;
 }
 
 Json& Json::operator=(Json&& other)
@@ -179,16 +167,14 @@ Json& Json::operator=(Json&& other)
     {
         return *this;
     }
-    if (allocated_)
-    {
-        destroy_node(node_);
-    }
+    destroy_node(&node_);
     node_ = other.node_;
     other.node_ = nullptr;
     parent_ = other.parent_;
     other.parent_ = nullptr;
-    parent_key_ = std::move(other.parent_key_);
     allocated_ = other.allocated_;
+    other.allocated_ = false;
+    parent_key_ = std::move(other.parent_key_);
     return *this;
 }
 
@@ -208,18 +194,18 @@ Json Json::parse(FILE* fp)
 {
     if (fp == nullptr)
     {
-        return Json(Empty());
+        return Json();
     }
     fseek(fp, 0, SEEK_END);
     long length = ftell(fp);
     fseek(fp, 0, SEEK_SET);
     char* buffer = (char*)malloc(length + 1);
     buffer[length] = '\0';
-    size_t ret = fread(buffer, 1, length, fp);
+    long ret = fread(buffer, 1, length, fp);
     Json js;
     if (ret != length)
     {
-        js = Json(Empty());
+        js = Json();
     }
     else
     {
@@ -244,10 +230,6 @@ std::string Json::dump(int spaces) const
 
 Json Json::operator[](const char* key)
 {
-    if (!is_valid())
-    {
-        return Json(Empty());
-    }
     if (is_null() && is_root())
     {
         // todo : need is_root here?
@@ -260,38 +242,40 @@ Json Json::operator[](const char* key)
         const json_value_t* res = json_object_find(key, obj);
         if (res != nullptr)
         {
-            return Json(res, node_, key);
+            return Json(res, node_, std::string(key));
         }
     }
     if (is_placeholder())
     {
-        destroy_node(node_);
+        destroy_node(&node_);
         json_object_t* parent_obj = json_value_object(parent_);
-        node_ = json_object_append(parent_obj, parent_key_.c_str(),
-                                   JSON_VALUE_OBJECT);
+        node_ = const_cast<json_value_t*>(json_object_append(
+            parent_obj, parent_key_.c_str(), JSON_VALUE_OBJECT));
     }
     if (!is_object())
     {
-        return Json(Empty());
+        return Json();
     }
     // (null, parent(node_), key)
-    return Json(node_, key);
+    Json js = Json();
+    js.set_parent(node_, std::string(key));
+    return js;
 }
 
 Json Json::operator[](const char* key) const
 {
-    if (!is_valid() || !is_object())
+    if (!is_object())
     {
-        return Json(Empty());
+        return Json();
     }
     const json_value_t* val = node_;
     json_object_t* obj = json_value_object(val);
     const json_value_t* res = json_object_find(key, obj);
     if (res != nullptr)
     {
-        return Json(res, node_);
+        return Json(res, node_, "");
     }
-    return Json(Empty());
+    return Json();
 }
 
 Json Json::operator[](const std::string& key)
@@ -327,7 +311,7 @@ Json Json::operator[](int index)
 {
     if (!is_array() || index < 0 || index > this->size())
     {
-        return Json(Empty());
+        return Json();
     }
     const json_value_t* val;
     json_array_t* arr = json_value_array(node_);
@@ -335,11 +319,11 @@ Json Json::operator[](int index)
     {
         if (index == 0)
         {
-            return Json(val, node_);
+            return Json(val, node_, "");
         }
         index--;
     }
-    return Json(Empty());
+    return Json();
 }
 
 void Json::erase(int index)
@@ -362,7 +346,7 @@ Json Json::operator[](int index) const
 {
     if (!is_array() || index < 0 || index > this->size())
     {
-        return Json(Empty());
+        return Json();
     }
     const json_value_t* val;
     json_array_t* arr = json_value_array(node_);
@@ -370,19 +354,15 @@ Json Json::operator[](int index) const
     {
         if (index == 0)
         {
-            return Json(val, node_);
+            return Json(val, node_, "");
         }
         index--;
     }
-    return Json(Empty());
+    return Json();
 }
 
 bool Json::can_obj_push_back()
 {
-    if (is_incomplete())
-    {
-        return false;
-    }
     if (is_placeholder() ||
         (parent_ != nullptr && json_value_type(parent_) == JSON_VALUE_OBJECT))
     {
@@ -397,6 +377,11 @@ bool Json::can_obj_push_back()
 
 void Json::push_back(const std::string& key, bool val)
 {
+    if (is_placeholder())
+    {
+        *this = Json::Object{{key, val}};
+        return;
+    }
     if (!can_obj_push_back())
     {
         return;
@@ -408,6 +393,11 @@ void Json::push_back(const std::string& key, bool val)
 
 void Json::push_back(const std::string& key, std::nullptr_t val)
 {
+    if (is_placeholder())
+    {
+        *this = Json::Object{{key, val}};
+        return;
+    }
     if (!can_obj_push_back())
     {
         return;
@@ -423,6 +413,11 @@ void Json::push_back(const std::string& key, const std::string& val)
 
 void Json::push_back(const std::string& key, const char* val)
 {
+    if (is_placeholder())
+    {
+        *this = Json::Object{{key, val}};
+        return;
+    }
     if (!can_obj_push_back())
     {
         return;
@@ -431,37 +426,66 @@ void Json::push_back(const std::string& key, const char* val)
     json_object_append(obj, key.c_str(), JSON_VALUE_STRING, val);
 }
 
-void Json::push_back(const std::string& key, const Json& val)
+void Json::push_back(const std::string& key, const std::vector<std::string>& val)
 {
+    if (is_placeholder())
+    {
+        *this = Json::Object{{key, val}};
+        return;
+    }
     if (!can_obj_push_back())
     {
         return;
     }
     json_object_t* obj = json_value_object(node_);
-    Json copy_json = val.copy();
+    const json_value_t* v = json_object_append(obj, key.c_str(),
+                                               JSON_VALUE_ARRAY);
+    json_array_t* arr = json_value_array(v);
+    for (const auto& str : val)
+    {
+        json_array_append(arr, JSON_VALUE_STRING, str.c_str());
+    }
+}
+
+void Json::push_back(const std::string& key, const Json& val)
+{
+    if (is_placeholder())
+    {
+        *this = Json::Object{{key, val}};
+        return;
+    }
+    if (!can_obj_push_back())
+    {
+        return;
+    }
+    json_object_t* obj = json_value_object(node_);
+    Json copy_json = val;
     json_object_append(obj, key.c_str(), 0, copy_json.node_);
-    copy_json.node_ = nullptr;
+    copy_json.reset();
 }
 
 void Json::placeholder_push_back(const std::string& key, bool val)
 {
     json_object_t* obj = json_value_object(parent_);
-    destroy_node(node_);
+    destroy_node(&node_);
     if (val)
     {
-        node_ = json_object_append(obj, key.c_str(), JSON_VALUE_TRUE);
+        node_ = const_cast<json_value_t*>(
+            json_object_append(obj, key.c_str(), JSON_VALUE_TRUE));
     }
     else
     {
-        node_ = json_object_append(obj, key.c_str(), JSON_VALUE_FALSE);
+        node_ = const_cast<json_value_t*>(
+            json_object_append(obj, key.c_str(), JSON_VALUE_FALSE));
     }
 }
 
 void Json::placeholder_push_back(const std::string& key, std::nullptr_t val)
 {
     json_object_t* obj = json_value_object(parent_);
-    destroy_node(node_);
-    node_ = json_object_append(obj, key.c_str(), JSON_VALUE_NULL);
+    destroy_node(&node_);
+    node_ = const_cast<json_value_t*>(
+        json_object_append(obj, key.c_str(), JSON_VALUE_NULL));
 }
 
 void Json::placeholder_push_back(const std::string& key, const std::string& val)
@@ -472,17 +496,31 @@ void Json::placeholder_push_back(const std::string& key, const std::string& val)
 void Json::placeholder_push_back(const std::string& key, const char* val)
 {
     json_object_t* obj = json_value_object(parent_);
-    destroy_node(node_);
-    node_ = json_object_append(obj, key.c_str(), JSON_VALUE_STRING, val);
+    destroy_node(&node_);
+    node_ = const_cast<json_value_t*>(
+        json_object_append(obj, key.c_str(), JSON_VALUE_STRING, val));
+}
+
+void Json::placeholder_push_back(const std::string& key,
+                                 const std::vector<std::string>& val)
+{
+    json_object_t* obj = json_value_object(parent_);
+    destroy_node(&node_);
+    node_ = const_cast<json_value_t*>(
+        json_object_append(obj, key.c_str(), JSON_VALUE_ARRAY));
+    json_array_t* arr = json_value_array(node_);
+    for (const auto& str : val)
+        json_array_append(arr, JSON_VALUE_STRING, str.c_str());
 }
 
 void Json::placeholder_push_back(const std::string& key, const Json& val)
 {
     json_object_t* obj = json_value_object(parent_);
-    destroy_node(node_);
-    Json copy_json = val.copy();
-    node_ = json_object_append(obj, key.c_str(), 0, copy_json.node_);
-    copy_json.node_ = nullptr;
+    destroy_node(&node_);
+    Json copy_json = val;
+    node_ = const_cast<json_value_t*>(
+        json_object_append(obj, key.c_str(), 0, copy_json.node_));
+    copy_json.reset();
 }
 
 void Json::normal_push_back(const std::string& key, bool val)
@@ -533,11 +571,33 @@ void Json::normal_push_back(const std::string& key, const char* val)
     json_value_destroy(remove_val);
 }
 
+void Json::normal_push_back(const std::string& key,
+                            const std::vector<std::string>& val)
+{
+    json_object_t* obj = json_value_object(parent_);
+    const json_value_t* find = json_object_find(key.c_str(), obj);
+    const json_value_t* v;
+    if (find == nullptr)
+    {
+        v = json_object_append(obj, key.c_str(), JSON_VALUE_ARRAY);
+    }
+    else
+    {
+        v = json_object_insert_before(find, obj, key.c_str(),
+                                      JSON_VALUE_ARRAY);
+        json_value_t* remove_val = json_object_remove(find, obj);
+        json_value_destroy(remove_val);
+    }
+    json_array_t* arr = json_value_array(v);
+    for (const auto& str : val)
+        json_array_append(arr, JSON_VALUE_STRING, str.c_str());
+}
+
 void Json::normal_push_back(const std::string& key, const Json& val)
 {
     json_object_t* obj = json_value_object(parent_);
     const json_value_t* find = json_object_find(key.c_str(), obj);
-    Json copy_json = val.copy();
+    Json copy_json = val;
     if (find == nullptr)
     {
         json_object_append(obj, key.c_str(), 0, copy_json.node_);
@@ -545,7 +605,7 @@ void Json::normal_push_back(const std::string& key, const Json& val)
         return;
     }
     json_object_insert_before(find, obj, key.c_str(), 0, copy_json.node_);
-    copy_json.node_ = nullptr;
+    copy_json.reset();
     json_value_t* remove_val = json_object_remove(find, obj);
     json_value_destroy(remove_val);
 }
@@ -561,11 +621,16 @@ bool Json::can_arr_push_back()
 
 Json Json::copy() const
 {
-    return Json(json_value_copy(node_), nullptr);
+    return *this;
 }
 
 void Json::push_back(bool val)
 {
+    if (is_placeholder())
+    {
+        *this = Json::Array{{val}};
+        return;
+    }
     if (!can_arr_push_back())
     {
         return;
@@ -575,23 +640,13 @@ void Json::push_back(bool val)
     json_array_append(arr, type);
 }
 
-void Json::push_back(const std::string& val)
-{
-    push_back(val.c_str());
-}
-
-void Json::push_back(const char* val)
-{
-    if (!can_arr_push_back())
-    {
-        return;
-    }
-    json_array_t* arr = json_value_array(node_);
-    json_array_append(arr, JSON_VALUE_STRING, val);
-}
-
 void Json::push_back(std::nullptr_t val)
 {
+    if (is_placeholder())
+    {
+        *this = Json::Array{{val}};
+        return;
+    }
     if (!can_arr_push_back())
     {
         return;
@@ -600,16 +655,48 @@ void Json::push_back(std::nullptr_t val)
     json_array_append(arr, JSON_VALUE_NULL);
 }
 
-void Json::push_back(const Json& val)
+void Json::push_back(const std::string& val)
 {
+    push_back(val.c_str());
+}
+void Json::push_back(const std::vector<std::string>& val)
+{
+    for (const auto& str : val)
+    {
+        push_back(str.c_str());
+    }
+}
+
+void Json::push_back(const char* val)
+{
+    if (is_placeholder())
+    {
+        *this = Json::Array{{val}};
+        return;
+    }
     if (!can_arr_push_back())
     {
         return;
     }
     json_array_t* arr = json_value_array(node_);
-    Json copy_json = val.copy();
+    json_array_append(arr, JSON_VALUE_STRING, val);
+}
+
+void Json::push_back(const Json& val)
+{
+    if (is_placeholder())
+    {
+        *this = Json::Array{{val}};
+        return;
+    }
+    if (!can_arr_push_back())
+    {
+        return;
+    }
+    json_array_t* arr = json_value_array(node_);
+    Json copy_json = val;
     json_array_append(arr, 0, copy_json.node_);
-    copy_json.node_ = nullptr;
+    copy_json.reset();
 }
 
 void Json::update_arr(bool val)
@@ -621,6 +708,14 @@ void Json::update_arr(bool val)
     json_value_destroy(remove_val);
 }
 
+void Json::update_arr(std::nullptr_t val)
+{
+    json_array_t* arr = json_value_array(parent_);
+    json_array_insert_before(node_, arr, JSON_VALUE_NULL);
+    json_value_t* remove_val = json_array_remove(node_, arr);
+    json_value_destroy(remove_val);
+}
+
 void Json::update_arr(const std::string& val)
 {
     update_arr(val.c_str());
@@ -634,20 +729,25 @@ void Json::update_arr(const char* val)
     json_value_destroy(remove_val);
 }
 
-void Json::update_arr(std::nullptr_t val)
+void Json::update_arr(const std::vector<std::string>& val)
 {
     json_array_t* arr = json_value_array(parent_);
-    json_array_insert_before(node_, arr, JSON_VALUE_NULL);
+    const json_value_t* v = json_array_insert_before(node_, arr, JSON_VALUE_ARRAY);
     json_value_t* remove_val = json_array_remove(node_, arr);
     json_value_destroy(remove_val);
+    arr = json_value_array(v);
+    for (const auto& str : val)
+    {
+        json_array_append(arr, JSON_VALUE_STRING, str.c_str());
+    }
 }
 
 void Json::update_arr(const Json& val)
 {
     json_array_t* arr = json_value_array(parent_);
-    Json copy_json = val.copy();
+    Json copy_json = val;
     json_array_insert_before(node_, arr, 0, copy_json.node_);
-    copy_json.node_ = nullptr;
+    copy_json.reset();
     json_value_t* remove_val = json_array_remove(node_, arr);
     json_value_destroy(remove_val);
 }
@@ -711,29 +811,50 @@ bool Json::empty() const
 
 void Json::clear()
 {
-    if (allocated_)
+    int type = json_value_type(node_);
+    destroy_node(&node_);
+    parent_ = nullptr;
+    allocated_ = true;
+    parent_key_.clear();
+    if (type == JSON_VALUE_STRING)
     {
-        destroy_node(node_);
-        node_ = json_value_create(JSON_VALUE_OBJECT);
+        node_ = json_value_create(JSON_VALUE_STRING, "");
+    }
+    else if (type == JSON_VALUE_NUMBER)
+    {
+
+        node_ = json_value_create(JSON_VALUE_NUMBER, 0.0);
+    }
+    else
+    {
+        node_ = json_value_create(type);
     }
 }
 
-void Json::to_object()
+bool Json::to_object()
 {
-    if (allocated_ && is_null())
+    if (!allocated_ || !is_null())
     {
-        destroy_node(node_);
-        node_ = json_value_create(JSON_VALUE_OBJECT);
+        // watcher and non-null type can't change type
+        return false;
     }
+    destroy_node(&node_);
+    node_ = json_value_create(JSON_VALUE_OBJECT);
+    allocated_ = true;
+    return true;
 }
 
-void Json::to_array()
+bool Json::to_array()
 {
-    if (allocated_ && is_null())
+    if (!allocated_ || !is_null())
     {
-        destroy_node(node_);
-        node_ = json_value_create(JSON_VALUE_ARRAY);
+        // watcher and non-null type can't change type
+        return false;
     }
+    destroy_node(&node_);
+    node_ = json_value_create(JSON_VALUE_ARRAY);
+    allocated_ = true;
+    return true;
 }
 
 void Json::value_convert(const json_value_t* val, int spaces, int depth,
