@@ -15,10 +15,13 @@
 */
 
 #include <stdio.h>
+#include "workflow/WFFacilities.h"
 #include "echo_pb.srpc.h"
 
 using namespace srpc;
 
+static WFFacilities::WaitGroup wait_group(1);
+
 int main()
 {
 	Example::BRPCClient client("127.0.0.1", 1412);
@@ -40,10 +43,7 @@ int main()
 
 		while (ctx->get_attachment(&attachment, &len))
 			printf("get attachment [%.*s] len=%zu\n", (int)len, attachment, len);
-
-		printf("no attachment\n\n");
-
-		});
+	});
 
 	//sync
 	EchoRequest sync_req;
@@ -61,6 +61,7 @@ int main()
 		printf("status[%d] error[%d] errmsg:%s\n",
 				sync_ctx.status_code, sync_ctx.error, sync_ctx.errmsg.c_str());
 
+	wait_group.wait();
 	return 0;
 }
 
